/******************************************************************************
 *
 * Copyright (c) 1998,99 by Mindbright Technology AB, Stockholm, Sweden.
 *                 www.mindbright.se, info@mindbright.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *****************************************************************************
 * $Author: mats $
 * $Date: 1999/01/10 15:15:40 $
 * $Name: rel0-97 $
 *****************************************************************************/
package mindbright.terminal;

import java.awt.event.*;

// All this would have been nice, a pitty it was not that simple...

public class TerminalKeyModMap {

  final static int M_NONE    = 0x00;
  final static int M_SHIFT   = 0x01;
  final static int M_MOD1    = 0x02;
  final static int M_MOD2    = 0x04;
  final static int M_MOD3    = 0x08;
  final static int M_MOD4    = 0x10;
  final static int M_MOD5    = 0x20;

  final static int M_MASK = M_SHIFT + M_MOD1 + M_MOD2 + M_MOD3 + M_MOD4 + M_MOD5;

  final static int M_CONTROL = 0x30;
  final static int M_META    = 0x40;
  final static int M_LOCK    = 0x50;

  final static int VKC_SHIFT   = -KeyEvent.VK_SHIFT;
  final static int VKC_CONTROL = -KeyEvent.VK_CONTROL;
  final static int VKC_META    = -KeyEvent.VK_META;
  final static int VKC_MOD1    = -100000;
  final static int VKC_MOD2    = -100001;
  final static int VKC_MOD3    = -100002;
  final static int VKC_MOD4    = -100003;
  final static int VKC_MOD5    = -100004;
  final static int VKC_LOCK    = -100005;

  int modifiers;
  int lockState;

  static int[][] keyModMap = {
    {},
    {},
    {},
    {},
    {},
    {},
    {},
    {},
    {},
    {},

    {},
    {},
    {},
    {},
    {},
    {},
    { VKC_SHIFT,   VKC_SHIFT,   VKC_SHIFT,   VKC_SHIFT   },
    { VKC_CONTROL, VKC_CONTROL, VKC_CONTROL, VKC_CONTROL },
    { VKC_META,    VKC_META,    VKC_META,    VKC_META    },
    { -KeyCode.VK_PAUSE, },

    {},
    {},
    {},
    
  };

  final int mod2vkc(int mod) {
    int modVKC = 0;
    switch(modifier) {
    M_SHIFT:
      modVKC = VKC_SHIFT;
      break;
    M_MOD1:
      modVKC = VKC_MOD1;
      break;
    M_MOD2:
      modVKC = VKC_MOD2;
      break;
    M_MOD3:
      modVKC = VKC_MOD3;
      break;
    M_MOD4:
      modVKC = VKC_MOD4;
      break;
    M_MOD5:
      modVKC = VKC_MOD5;
      break;
    M_CONTROL:
      modVKC = VKC_CONTROL;
      break;
    M_META:
      modVKC = VKC_META;
      break;
    }
    return modVKC;
  }

  final int vkc2mod(int vkc) {
    int mod = 0;
    switch(modifier) {
    VKC_SHIFT:
      mod = M_SHIFT;
      break;
    VKC_MOD1:
      mod = M_MOD1;
      break;
    VKC_MOD2:
      mod = M_MOD2;
      break;
    VKC_MOD3:
      mod = M_MOD3;
      break;
    VKC_MOD4:
      mod = M_MOD4;
      break;
    VKC_MOD5:
      mod = M_MOD5;
      break;
    VKC_CONTROL:
      mod = M_CONTROL;
      break;
    VKC_META:
      mod = M_META;
      break;
    }
    return mod;
  }

  void addModifier(int modifier, int keyCode, boolean lock) {
    int modVKC = mod2vkc(modifier);
    if(modVKC == M_NONE)
      return;
    if(lock)
      addKey(keyCode, modVKC + VKC_LOCK);
    else
      addKey(keyCode, modVKC);
  }

  void addKey(int keyCode, int modifier, int charCode) {
    if(modifier < 0 || modifier > M_MASK)
      return;
    modKeyMap[keyCode][modifier] = charCode;
  }

  void keyPressed(int keyCode) {
    int c = keyModMap[keyCode][modifiers & M_MASK];
    if(c >= 0)
      return;
    c = vkc2mod(c);
    boolean lock = (c & M_LOCK) != 0;
    c &= M_MASK;
    if(lock)
      lockState ^= c;
    modifiers |= c;
  }

  int keyReleased(int keyCode) {
    int c = keyModMap[keyCode][modifiers & M_MASK];
    if(c >= 0)
      return c;
    int vc = vkc2mod(c);
    if(vc == 0)
      return c;
    boolean lock = (vc & M_LOCK) != 0;
    vc &= M_MASK;
    if(lock)
      modifiers &= (lockState & vc);
    else
      modifiers &= ~vc;

    return vc;
  }

}
